@@ -15,16 +15,15 @@
 
 Well, at least begin testing some of the functionality
 
-$Id: testZopeGuards.py,v 1.1.4.9 2005/04/05 21:17:44 tseaver Exp $
+$Id: testZopeGuards.py 37653 2005-08-02 09:37:08Z andreasjung $
 """
 
-
 import os, sys
 import unittest
 import ZODB
 import AccessControl.SecurityManagement
 from AccessControl.SimpleObjectPolicies import ContainerAssertions
-from zExceptions import Unauthorized
+from AccessControl import Unauthorized
 from AccessControl.ZopeGuards \
     import guarded_getattr, get_dict_get, get_dict_pop, get_list_pop, \
     get_iter, guarded_min, guarded_max, safe_builtins, guarded_enumerate, \
@@ -78,64 +77,48 @@ class Method:
         self.args = args
 
 
-class TestGuardedGetattrPy(GuardTestCase):
+class TestGuardedGetattr(GuardTestCase):
 
     def setUp(self):
-        import AccessControl.ImplPython
-        self.guarded_getattr = AccessControl.ImplPython.guarded_getattr
         self.__sm = SecurityManager()
         self.__old = self.setSecurityManager(self.__sm)
 
     def tearDown(self):
         self.setSecurityManager(self.__old)
 
+    def test_unauthorized(self):
+        obj, name = Method(), 'args'
+        value = getattr(obj, name)
+        rc = sys.getrefcount(value)
+        self.__sm.reject = True
+        self.assertRaises(Unauthorized, guarded_getattr, obj, name)
+        self.assert_(self.__sm.calls)
+        del self.__sm.calls[:]
+        self.assertEqual(rc, sys.getrefcount(value))
+
     def test_calls_validate_for_unknown_type(self):
-        self.guarded_getattr(self, 'test_calls_validate_for_unknown_type')
+        guarded_getattr(self, 'test_calls_validate_for_unknown_type')
         self.assert_(self.__sm.calls)
 
     def test_attr_handler_table(self):
-
         d = {}
         _dict = type(d)
         old = ContainerAssertions.get(_dict)
 
-        def myfactory(name, value):
-            if name == 'values':
-                return Method
-            return 1
-
-        ContainerAssertions[_dict] = myfactory
-
+        mytable = {'keys': 1,
+                   'values': Method,
+                   }
+        ContainerAssertions[_dict] = mytable
         try:
-            self.guarded_getattr(d, 'keys')
+            guarded_getattr(d, 'keys')
             self.assertEqual(len(self.__sm.calls), 0)
-            values = self.guarded_getattr(d, 'values')
+            values = guarded_getattr(d, 'values')
             self.assertEqual(values.__class__, Method)
             self.assertEqual(values.args, (d, 'values'))
+            self.assertRaises(Unauthorized, guarded_getattr, d, 'items')
         finally:
             ContainerAssertions[_dict] = old
 
-    def test_with_unicode(self):
-        # test for Collector #1617
-        d = {u'\xcf' : u'\xfc',
-             '\xab' : u'\xcd'}
-        if sys.getdefaultencoding() == 'ascii':
-            try:
-                self.guarded_getattr(d, u'\xcf')
-            except UnicodeEncodeError:
-                pass 
-        else:
-            try:
-                self.guarded_getattr(d, '\xab')
-            except AttributeError:
-                pass
-
-class TestGuardedGetattrC(TestGuardedGetattrPy):
-
-    def setUp(self):
-        TestGuardedGetattrPy.setUp(self)
-        import AccessControl.ImplC
-        self.guarded_getattr = AccessControl.ImplC.guarded_getattr
 
 class TestDictGuards(GuardTestCase):
 
@@ -599,6 +582,7 @@ print sneaky.private_method()
         else:
             self.fail("Didn't raise SyntaxError!")
 
+
     def test_dict_access(self):
         from RestrictedPython.tests import verify
 
@@ -624,7 +608,7 @@ print foo(**kw)
 
         self.assertEqual(its_globals['_print'](),
                         'baz\nTrue\n')
-
+        
     def _compile_str(self, text, name):
         from RestrictedPython import compile_restricted
         from AccessControl.ZopeGuards import get_safe_globals, guarded_getattr
@@ -637,6 +621,10 @@ print foo(**kw)
         g['__name__'] = __name__ # so classes can be defined in the script
         return code, g
 
+    def testPythonRealAC(self):
+        code, its_globals = self._compile("actual_python.py")
+        exec code in its_globals
+
     # Compile code in fname, as restricted Python. Return the
     # compiled code, and a safe globals dict for running it in.
     # fname is the string name of a Python file; it must be found
@@ -661,8 +649,7 @@ print foo(**kw)
 
 def test_suite():
     suite = unittest.TestSuite()
-    for cls in (TestGuardedGetattrPy,
-                TestGuardedGetattrC,
+    for cls in (TestGuardedGetattr,
                 TestDictGuards,
                 TestBuiltinFunctionGuards,
                 TestListGuards,
