@@ -10,10 +10,10 @@
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-"""Image object"""
-
-__version__='$Revision: 1.145.2.12 $'[11:-2]
+"""Image object
 
+$Id: Image.py 33280 2005-07-12 15:02:15Z tiran $
+"""
 import Globals, struct
 from OFS.content_types import guess_content_type
 from Globals import DTMLFile
@@ -34,6 +34,7 @@ from ZPublisher.HTTPRequest import FileUpload
 from ZPublisher.Iterators import filestream_iterator
 from zExceptions import Redirect
 from cgi import escape
+import transaction
 
 StringType=type('')
 manage_addFileForm=DTMLFile('dtml/imageAdd', globals(),Kind='File',kind='file')
@@ -515,11 +516,9 @@ class File(Persistent, Implicit, PropertyManager,
 
         # Make sure we have an _p_jar, even if we are a new object, by
         # doing a sub-transaction commit.
-        get_transaction().commit(1)
+        transaction.savepoint()
 
-        jar=self._p_jar
-
-        if jar is None:
+        if self._p_jar is None:
             # Ugh
             seek(0)
             return Pdata(read(size)), size
@@ -528,31 +527,29 @@ class File(Persistent, Implicit, PropertyManager,
         # to front to minimize the number of database updates
         # and to allow us to get things out of memory as soon as
         # possible.
-        next=None
+        next = None
         while end > 0:
-            pos=end-n
-            if pos < n: pos=0 # we always want at least n bytes
+            pos = end-n
+            if pos < n:
+                pos = 0 # we always want at least n bytes
             seek(pos)
-            data=Pdata(read(end-pos))
+            
+            # Create the object and assign it a next pointer
+            # in the same transaction, so that there is only
+            # a single database update for it.
+            data = Pdata(read(end-pos))
+            self._p_jar.add(data)
+            data.next = next
 
-            # Woooop Woooop Woooop! This is a trick.
-            # We stuff the data directly into our jar to reduce the
-            # number of updates necessary.
-            data._p_jar=jar
+            # Save the object so that we can release its memory.
+            transaction.savepoint()
+            data._p_deactivate()
+            # The object should be assigned an oid and be a ghost.
+            assert data._p_oid is not None
+            assert data._p_state == -1
 
-            # This is needed and has side benefit of getting
-            # the thing registered:
-            data.next=next
-
-            # Now make it get saved in a sub-transaction!
-            get_transaction().commit(1)
-
-            # Now make it a ghost to free the memory.  We
-            # don't need it anymore!
-            data._p_changed=None
-
-            next=data
-            end=pos
+            next = data
+            end = pos
 
         return next, size
 
@@ -609,6 +606,7 @@ class File(Persistent, Implicit, PropertyManager,
                 return result
         return str(self.data)
 
+
 manage_addImageForm=DTMLFile('dtml/imageAdd',globals(),
                              Kind='Image',kind='image')
 def manage_addImage(self, id, file, title='', precondition='', content_type='',
@@ -821,7 +819,7 @@ class Image(File):
         if width:
             result = '%s width="%s"' % (result, width)
 
-         # Collector #1557
+        # Omitting 'border' attribute (Collector #1557)
 #        if not 'border' in [ x.lower() for x in  args.keys()]:
 #            result = '%s border="0"' % result
 
@@ -872,4 +870,3 @@ class Pdata(Persistent, Implicit):
             next=self.next
 
         return ''.join(r)
-
