@@ -17,13 +17,21 @@ Page Template-specific implementation of TALES, with handlers
 for Python expressions, string literals, and paths.
 """
 
-__version__='$Revision: 1.43.44.5 $'[11:-2]
+__version__='$Revision: 1.45 $'[11:-2]
 
 import re, sys
-from TALES import Engine, CompilerError, _valid_name, NAME_RE, \
-     Undefined, Default, _parse_expr
+from TALES import Engine
+from TALES import CompilerError
+from TALES import _valid_name
+from TALES import NAME_RE
+from TALES import Undefined
+from TALES import Default
+from TALES import _parse_expr
 from Acquisition import aq_base, aq_inner, aq_parent
-
+from DeferExpr import LazyWrapper
+from DeferExpr import LazyExpr
+from DeferExpr import DeferWrapper
+from DeferExpr import DeferExpr
 
 _engine = None
 def getEngine():
@@ -43,6 +51,7 @@ def installHandlers(engine):
     reg('python', PythonExpr)
     reg('not', NotExpr)
     reg('defer', DeferExpr)
+    reg('lazy', LazyExpr)
 
 import AccessControl
 import AccessControl.cAccessControl
@@ -50,7 +59,9 @@ acquisition_security_filter = AccessControl.cAccessControl.aq_validate
 from AccessControl import getSecurityManager
 from AccessControl.ZopeGuards import guarded_getattr
 from AccessControl import Unauthorized
-from ZRPythonExpr import PythonExpr, _SecureModuleImporter, call_with_ns
+from ZRPythonExpr import PythonExpr
+from ZRPythonExpr import _SecureModuleImporter
+from ZRPythonExpr import call_with_ns
 
 SecureModuleImporter = _SecureModuleImporter()
 
@@ -235,29 +246,6 @@ class NotExpr:
     def __repr__(self):
         return 'not:%s' % `self._s`
 
-class DeferWrapper:
-    def __init__(self, expr, econtext):
-        self._expr = expr
-        self._econtext = econtext
-
-    def __str__(self):
-        return str(self())
-
-    def __call__(self):
-        return self._expr(self._econtext)
-
-class DeferExpr:
-    def __init__(self, name, expr, compiler):
-        self._s = expr = expr.lstrip()
-        self._c = compiler.compile(expr)
-
-    def __call__(self, econtext):
-        return DeferWrapper(self._c, econtext)
-
-    def __repr__(self):
-        return 'defer:%s' % `self._s`
-
-
 def restrictedTraverse(object, path, securityManager,
                        get=getattr, has=hasattr, N=None, M=[],
                        TupleType=type(()) ):
@@ -278,7 +266,7 @@ def restrictedTraverse(object, path, securityManager,
             # Skip directly to item access
             o = object[name]
             # Check access to the item.
-            if not validate(object, object, name, o):
+            if not validate(object, object, None, o):
                 raise Unauthorized, name
             object = o
             continue
@@ -333,7 +321,7 @@ def restrictedTraverse(object, path, securityManager,
                     raise
                 else:
                     # Check access to the item.
-                    if not validate(object, object, name, o):
+                    if not validate(object, object, None, o):
                         raise Unauthorized, name
         object = o
 
