@@ -7,28 +7,31 @@
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+"""Date index.
 
-"""$Id: DateIndex.py,v 1.12.2.4 2005/03/18 15:31:01 andreasjung Exp $
+$Id: DateIndex.py 30994 2005-07-04 16:59:17Z yuppie $
 """
 
+import time
+from datetime import date, datetime
 from datetime import tzinfo, timedelta
 from types import StringType, FloatType, IntType
-from DateTime.DateTime import DateTime
-from OFS.PropertyManager import PropertyManager
-from datetime import date, datetime
-from Products.PluginIndexes import PluggableIndex
-from Products.PluginIndexes.common.UnIndex import UnIndex
-from Products.PluginIndexes.common.util import parseIndexRequest
-from Products.PluginIndexes.common import safe_callable
 
-from Globals import DTMLFile
+from BTrees.IIBTree import IISet, union, intersection, multiunion
 from BTrees.IOBTree import IOBTree
 from BTrees.OIBTree import OIBTree
-from BTrees.IIBTree import IISet, union, intersection, multiunion
-import time
+from DateTime.DateTime import DateTime
+from Globals import DTMLFile
+from OFS.PropertyManager import PropertyManager
+from zope.interface import implements
+
+from Products.PluginIndexes.common import safe_callable
+from Products.PluginIndexes.common.UnIndex import UnIndex
+from Products.PluginIndexes.common.util import parseIndexRequest
+from Products.PluginIndexes.interfaces import IDateIndex
 
 _marker = []
 
@@ -73,34 +76,40 @@ class LocalTimezone(tzinfo):
 Local = LocalTimezone()
 ###############################################################################
 
-class DateIndex(UnIndex, PropertyManager):
-    """ Index for Dates """
 
-    __implements__ = (PluggableIndex.UniqueValueIndex,
-                      PluggableIndex.SortIndex)
+class DateIndex(UnIndex, PropertyManager):
+
+    """Index for dates.
+    """
+
+    __implements__ = UnIndex.__implements__
+    implements(IDateIndex)
 
     meta_type = 'DateIndex'
     query_options = ['query', 'range']
-    
+
     index_naive_time_as_local = True # False means index as UTC
-    _properties=({'id':'index_naive_time_as_local', 
+    _properties=({'id':'index_naive_time_as_local',
                   'type':'boolean',
                   'mode':'w'},)
 
     manage = manage_main = DTMLFile( 'dtml/manageDateIndex', globals() )
+    manage_browse = DTMLFile('../dtml/browseIndex', globals())
+
     manage_main._setName( 'manage_main' )
     manage_options = ( { 'label' : 'Settings'
                        , 'action' : 'manage_main'
                        },
+                       {'label': 'Browse',
+                        'action': 'manage_browse',
+                       },
                      ) + PropertyManager.manage_options
 
     def clear( self ):
         """ Complete reset """
-        try: self.__len__.set(0)
-        except AttributeError: self.__len__=BTrees.Length.Length()
         self._index = IOBTree()
         self._unindex = OIBTree()
-
+        self._length.set(0)
 
     def index_object( self, documentId, obj, threshold=None ):
         """index an object, normalizing the indexed value to an integer
@@ -135,7 +144,6 @@ class DateIndex(UnIndex, PropertyManager):
 
         return returnStatus
 
-
     def _apply_index( self, request, cid='', type=type ):
         """Apply the index to query parameters given in the argument
 
@@ -217,7 +225,6 @@ class DateIndex(UnIndex, PropertyManager):
         else:
             return r, (self.id,)
 
-
     def _convert( self, value, default=None ):
         """Convert Date/Time value to our internal representation"""
         # XXX: Code patched 20/May/2003 by Kiran Jonnalagadda to
