@@ -7,39 +7,48 @@
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+"""Base for bi-directional indexes.
 
-"""Simple column indices"""
-
-__version__='$Revision: 1.20.2.8 $'[11:-2]
+$Id: UnIndex.py 31006 2005-07-05 13:57:35Z tseaver $
+"""
 
 import sys
 from cgi import escape
-from types import IntType
+from logging import getLogger
 
-from Globals import Persistent
-from Acquisition import Implicit
-from zLOG import LOG, ERROR
-
-from BTrees.OOBTree import OOBTree, OOSet
-from BTrees.IOBTree import IOBTree
 from BTrees.IIBTree import IITreeSet, IISet, union, intersection
-from OFS.SimpleItem import SimpleItem
+from BTrees.IOBTree import IOBTree
 import BTrees.Length
+from BTrees.OOBTree import OOBTree
+from OFS.SimpleItem import SimpleItem
 from ZODB.POSException import ConflictError
+from zope.interface import implements
 
-from Products.PluginIndexes.common.util import parseIndexRequest
+from Products.PluginIndexes import PluggableIndex
 from Products.PluginIndexes.common import safe_callable
+from Products.PluginIndexes.common.util import parseIndexRequest
+from Products.PluginIndexes.interfaces import IPluggableIndex
+from Products.PluginIndexes.interfaces import ISortIndex
+from Products.PluginIndexes.interfaces import IUniqueValueIndex
 
 _marker = []
-
-class UnIndex(Persistent, Implicit, SimpleItem):
-    """UnIndex object interface"""
+LOG = getLogger('Zope.UnIndex')
 
 
-    def __init__(self, id, ignore_ex=None, call_methods=None, extra=None, caller=None):
+class UnIndex(SimpleItem):
+
+    """Simple forward and reverse index.
+    """
+
+    __implements__ = (PluggableIndex.UniqueValueIndex,
+                      PluggableIndex.SortIndex)
+    implements(IPluggableIndex, IUniqueValueIndex, ISortIndex)
+
+    def __init__(
+        self, id, ignore_ex=None, call_methods=None, extra=None, caller=None):
         """Create an unindex
 
         UnIndexes are indexes that contain two index components, the
@@ -77,7 +86,6 @@ class UnIndex(Persistent, Implicit, SimpleItem):
 
           'caller' -- reference to the calling object (usually
           a (Z)Catalog instance
-
         """
 
         def _get(o, k, default):
@@ -101,69 +109,34 @@ class UnIndex(Persistent, Implicit, SimpleItem):
         else:
             self.indexed_attrs = list(ia)
         self.indexed_attrs = [ attr.strip()
-                                for attr in  self.indexed_attrs if attr ] 
+                               for attr in self.indexed_attrs if attr ]
         if not self.indexed_attrs:
             self.indexed_attrs = [id]
 
-        self.__len__ = BTrees.Length.Length() # see __len__ method docstring
+        self._length = BTrees.Length.Length()
         self.clear()
 
-    def getId(self): return self.id
+    def __len__(self):
+        return self._length()
+
+    def getId(self):
+        return self.id
 
     def clear(self):
-        # inplace opportunistic conversion from old-style to new style BTrees
-        try: self.__len__.set(0)
-        except AttributeError: self.__len__=BTrees.Length.Length()
+        self._length = BTrees.Length.Length()
         self._index = OOBTree()
         self._unindex = IOBTree()
 
-    def _convertBTrees(self, threshold=200):
-        if type(self._index) is OOBTree: return
-
-        from BTrees.convert import convert
-
-        _index=self._index
-        self._index=OOBTree()
-
-        def convertSet(s,
-                       IITreeSet=IITreeSet, IntType=type(0),
-                       type=type, len=len,
-                       doneTypes = (IntType, IITreeSet)):
-
-            if type(s) in doneTypes: return s
-
-            if len(s) == 1:
-                try: return s[0]  # convert to int
-                except ConflictError: raise
-                except: pass # This is just an optimization.
-
-            return IITreeSet(s)
-
-        convert(_index, self._index, threshold, convertSet)
-
-        _unindex=self._unindex
-        self._unindex=IOBTree()
-        convert(_unindex, self._unindex, threshold)
-
-        self.__len__=BTrees.Length.Length(len(_index))
-
     def __nonzero__(self):
         return not not self._unindex
 
-    def __len__(self):
-        """Return the number of objects indexed.
-
-        This method is overridden per-instance with a cached Length object
-        """
-        return len(self._unindex)
-
     def histogram(self):
         """Return a mapping which provides a histogram of the number of
-        elements found at each point in the index."""
-
+        elements found at each point in the index.
+        """
         histogram = {}
         for item in self._index.items():
-            if type(item) is IntType:
+            if isinstance(item,int):
                 entry = 1 # "set" length is 1
             else:
                 key, value = item
@@ -172,68 +145,64 @@ class UnIndex(Persistent, Implicit, SimpleItem):
 
         return histogram
 
-
     def referencedObjects(self):
         """Generate a list of IDs for which we have referenced objects."""
         return self._unindex.keys()
 
-
     def getEntryForObject(self, documentId, default=_marker):
         """Takes a document ID and returns all the information we have
-        on that specific object."""
+        on that specific object.
+        """
         if default is _marker:
             return self._unindex.get(documentId)
         else:
             return self._unindex.get(documentId, default)
 
-
     def removeForwardIndexEntry(self, entry, documentId):
         """Take the entry provided and remove any reference to documentId
-        in its entry in the index."""
-        global _marker
+        in its entry in the index.
+        """
         indexRow = self._index.get(entry, _marker)
         if indexRow is not _marker:
             try:
                 indexRow.remove(documentId)
                 if not indexRow:
                     del self._index[entry]
-                    try: self.__len__.change(-1)
-                    except AttributeError: pass # pre-BTrees-module instance
+                    self._length.change(-1)
+
             except ConflictError:
                 raise
+
             except AttributeError:
                 # index row is an int
                 del self._index[entry]
-                try: self.__len__.change(-1)
-                except AttributeError: pass # pre-BTrees-module instance
-            except:
-                LOG(self.__class__.__name__, ERROR,
-                    ('unindex_object could not remove '
-                     'documentId %s from index %s.  This '
-                     'should not happen.'
-                     % (str(documentId), str(self.id))), '',
-                    sys.exc_info())
-        else:
-            LOG(self.__class__.__name__, ERROR,
-                ('unindex_object tried to retrieve set %s '
-                 'from index %s but couldn\'t.  This '
-                 'should not happen.' % (repr(entry), str(self.id))))
+                self._length.change(-1)
 
+            except:
+                LOG.error('%s: unindex_object could not remove '
+                          'documentId %s from index %s.  This '
+                          'should not happen.' % (self.__class__.__name__,
+                           str(documentId), str(self.id)),
+                           exc_info=sys.exc_info())
+        else:
+            LOG.error('%s: unindex_object tried to retrieve set %s '
+                      'from index %s but couldn\'t.  This '
+                      'should not happen.' % (self.__class__.__name__,
+                      repr(entry), str(self.id)))
 
     def insertForwardIndexEntry(self, entry, documentId):
         """Take the entry provided and put it in the correct place
         in the forward index.
 
-        This will also deal with creating the entire row if necessary."""
-        global _marker
+        This will also deal with creating the entire row if necessary.
+        """
         indexRow = self._index.get(entry, _marker)
 
         # Make sure there's actually a row there already.  If not, create
         # an IntSet and stuff it in first.
         if indexRow is _marker:
             self._index[entry] = documentId
-            try:  self.__len__.change(1)
-            except AttributeError: pass # pre-BTrees-module instance
+            self._length.change(1)
         else:
             try: indexRow.insert(documentId)
             except AttributeError:
@@ -241,7 +210,6 @@ class UnIndex(Persistent, Implicit, SimpleItem):
                 indexRow=IITreeSet((indexRow, documentId))
                 self._index[entry] = indexRow
 
-
     def index_object(self, documentId, obj, threshold=None):
         """ wrapper to handle indexing of multiple attributes """
 
@@ -253,10 +221,8 @@ class UnIndex(Persistent, Implicit, SimpleItem):
 
         return res > 0
 
-
     def _index_object(self, documentId, obj, threshold=None, attr=''):
         """ index and object 'obj' with integer id 'documentId'"""
-        global _marker
         returnStatus = 0
 
         # First we need to see if there's anything interesting to look at
@@ -274,9 +240,8 @@ class UnIndex(Persistent, Implicit, SimpleItem):
                     except ConflictError:
                         raise
                     except:
-                        LOG('UnIndex', ERROR,
-                            'Should not happen: oldDatum was there, now its not,'
-                            'for document with id %s' % documentId)
+                        LOG.error('Should not happen: oldDatum was there, now its not,'
+                                  'for document with id %s' % documentId)
 
             if datum is not _marker:
                 self.insertForwardIndexEntry(datum, documentId)
@@ -300,14 +265,16 @@ class UnIndex(Persistent, Implicit, SimpleItem):
 
     def numObjects(self):
         """ return number of indexed objects """
-        return len(self)
+        return len(self._unindex)
 
+    def indexSize(self):
+        """ return of distinct values indexed"""
+        return len(self)
 
     def unindex_object(self, documentId):
         """ Unindex the object with integer id 'documentId' and don't
-        raise an exception if we fail """
-
-        global _marker
+        raise an exception if we fail
+        """
         unindexRecord = self._unindex.get(documentId, _marker)
         if unindexRecord is _marker:
             return None
@@ -319,8 +286,8 @@ class UnIndex(Persistent, Implicit, SimpleItem):
         except ConflictError:
             raise
         except:
-            LOG('UnIndex', ERROR, 'Attempt to unindex nonexistent document'
-                ' with id %s' % documentId)
+            LOG.error('Attempt to unindex nonexistent document'
+                      ' with id %s' % documentId)
 
     def _apply_index(self, request, cid='', type=type):
         """Apply the index to query parameters given in the request arg.
@@ -360,10 +327,7 @@ class UnIndex(Persistent, Implicit, SimpleItem):
         FAQ answer:  to search a Field Index for documents that
         have a blank string as their value, wrap the request value
         up in a tuple ala: request = {'id':('',)}
-
         """
-
-
         record = parseIndexRequest(request, self.id, self.query_options)
         if record.keys==None: return None
 
@@ -371,7 +335,6 @@ class UnIndex(Persistent, Implicit, SimpleItem):
         r     = None
         opr   = None
 
-
         # experimental code for specifing the operator
         operator = record.get('operator',self.useOperator)
         if not operator in self.operators :
@@ -391,13 +354,11 @@ class UnIndex(Persistent, Implicit, SimpleItem):
             if range_parm.find("max")>-1:
                 opr_args.append("max")
 
-
         if record.get('usage',None):
             # see if any usage params are sent to field
             opr = record.usage.lower().split(':')
             opr, opr_args=opr[0], opr[1:]
 
-
         if opr=="range":   # range search
             if 'min' in opr_args: lo = min(record.keys)
             else: lo = None
@@ -409,7 +370,7 @@ class UnIndex(Persistent, Implicit, SimpleItem):
                 setlist = index.items(lo)
 
             for k, set in setlist:
-                if type(set) is IntType:
+                if isinstance(set, int):
                     set = IISet((set,))
                 r = set_func(r, set)
         else: # not a range search
@@ -417,11 +378,11 @@ class UnIndex(Persistent, Implicit, SimpleItem):
                 set=index.get(key, None)
                 if set is None:
                     set = IISet(())
-                elif type(set) is IntType: # isinstance(set, int) for 2.8
+                elif isinstance(set, int):
                     set = IISet((set,))
                 r = set_func(r, set)
 
-        if type(r) is IntType:  r=IISet((r,))
+        if isinstance(r, int):  r=IISet((r,))
         if r is None:
             return IISet(), (self.id,)
         else:
@@ -436,11 +397,11 @@ class UnIndex(Persistent, Implicit, SimpleItem):
 
     def getIndexSourceNames(self):
         """ return sequence of indexed attributes """
+        # BBB:  older indexes didn't have 'indexed_attrs'
         return getattr(self, 'indexed_attrs', [self.id])
 
     def uniqueValues(self, name=None, withLengths=0):
-        """\
-        returns the unique values for name
+        """returns the unique values for name
 
         if withLengths is true, returns a sequence of
         tuples of (value, length)
@@ -456,7 +417,7 @@ class UnIndex(Persistent, Implicit, SimpleItem):
             rl=[]
             for i in self._index.keys():
                 set = self._index[i]
-                if type(set) is IntType:
+                if isinstance(set, int):
                     l = 1
                 else:
                     l = len(set)
@@ -473,7 +434,7 @@ class UnIndex(Persistent, Implicit, SimpleItem):
     def items(self):
         items = []
         for k,v in self._index.items():
-            if type(v) is IntType:
+            if isinstance(v, int):
                 v = IISet((v,))
             items.append((k, v))
         return items
