@@ -12,12 +12,14 @@
 ##############################################################################
 """ ZCatalog product
 
-$Id: ZCatalog.py,v 1.126.2.6 2004/05/26 06:47:07 andreasjung Exp $
+$Id: ZCatalog.py 33280 2005-07-12 15:02:15Z tiran $
 """
 
+from warnings import warn
+import urllib, time, sys, string, logging
+
 from Globals import DTMLFile, MessageDialog
 import Globals
-
 from OFS.Folder import Folder
 from OFS.ObjectManager import ObjectManager
 from DateTime import DateTime
@@ -26,20 +28,25 @@ from Persistence import Persistent
 from DocumentTemplate.DT_Util import InstanceDict, TemplateDict
 from DocumentTemplate.DT_Util import Eval
 from AccessControl.Permission import name_trans
-from Catalog import Catalog, CatalogError
 from AccessControl.DTML import RestrictedDTML
 from AccessControl.Permissions import \
     manage_zcatalog_entries, manage_zcatalog_indexes, search_zcatalog
-from ZCatalogIndexes import ZCatalogIndexes
 from ZODB.POSException import ConflictError
+import transaction
 from Products.PluginIndexes.common.PluggableIndex \
      import PluggableIndexInterface
 from Products.PluginIndexes.TextIndex import Splitter
-import urllib, time, sys
-import string, logging
-from IZCatalog import IZCatalog
-from zLOG import LOG, INFO, BLATHER, ERROR
+from zLOG import LOG
+from zope.interface import implements
 
+from Catalog import Catalog, CatalogError
+from interfaces import IZCatalog as z3IZCatalog
+from IZCatalog import IZCatalog as z2IZCatalog
+from ProgressHandler import ZLogHandler
+from ZCatalogIndexes import ZCatalogIndexes
+
+
+LOG = logging.getLogger('Zope.ZCatalog')
 
 manage_addZCatalogForm=DTMLFile('dtml/addZCatalog',globals())
 
@@ -73,10 +80,11 @@ class ZCatalog(Folder, Persistent, Implicit):
     instance of the Catalog class.  An interesting feature of this
     class is that it is not Zope specific.  You can use it in any
     Python program to catalog objects.
-
     """
 
-    __implements__ = IZCatalog
+    __implements__ = z2IZCatalog
+    implements(z3IZCatalog)
+
     meta_type = "ZCatalog"
     icon='misc_/ZCatalog/ZCatalog.gif'
 
@@ -127,6 +135,7 @@ class ZCatalog(Folder, Persistent, Implicit):
           'manage_catalogClear', 'manage_addColumn', 'manage_delColumn',
           'manage_addIndex', 'manage_delIndex', 'manage_clearIndex',
           'manage_reindexIndex', 'manage_main', 'availableSplitters',
+          'manage_setProgress',
 
           # these two are deprecated:
           'manage_delColumns', 'manage_deleteIndex'
@@ -181,6 +190,9 @@ class ZCatalog(Folder, Persistent, Implicit):
         self._catalog = Catalog()
 
     def __len__(self):
+        # Perform a migration of _catalog.__len__ to _catalog._length
+        # to avoid with new-style class caching issues (see #1332)
+        self._catalog.migrate__len__()
         return len(self._catalog)
 
 
@@ -250,7 +262,9 @@ class ZCatalog(Folder, Persistent, Implicit):
         elapse = time.time()
         c_elapse = time.clock()
 
-        self.refreshCatalog(clear=1)
+        pgthreshold = self._getProgressThreshold()
+        handler = (pgthreshold > 0) and ZLogHandler(pgthreshold) or None
+        self.refreshCatalog(clear=1, pghandler=handler)
 
         elapse = time.time() - elapse
         c_elapse = time.clock() - c_elapse
@@ -263,7 +277,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                          'Total CPU time: %s' % (`elapse`, `c_elapse`)))
 
 
-    def refreshCatalog(self, clear=0):
+    def refreshCatalog(self, clear=0, pghandler=None):
         """ re-index everything we can find """
 
         cat = self._catalog
@@ -272,26 +286,27 @@ class ZCatalog(Folder, Persistent, Implicit):
             paths = tuple(paths)
             cat.clear()
 
-        LOG('ZCatalog', BLATHER, 'Starting recataloging of ZCatalog at %s' % 
-             self.absolute_url(1))
         num_objects = len(paths)
+        if pghandler:
+            pghandler.init('Refreshing catalog: %s' % self.absolute_url(1), num_objects)
+
         for i in xrange(num_objects):
+            if pghandler: pghandler.report(i)
+
             p = paths[i]
             obj = self.resolve_path(p)
             if not obj:
                 obj = self.resolve_url(p, self.REQUEST)
             if obj is not None:
                 try:
-                    LOG('ZCatalog', BLATHER, 'Recataloging object %s (%d/%d)' %
-                         (p, i, num_objects))
-                    self.catalog_object(obj, p)
+                    self.catalog_object(obj, p, pghandler=pghandler)
                 except ConflictError:
                     raise
                 except:
-                    LOG('ZCatalog', ERROR, 'Recataloging object at %s failed' % p,
-                              error=sys.exc_info())
+                    LOG.error('Recataloging object at %s failed' % p,
+                              exc_info=sys.exc_info())
 
-        LOG('ZCatalog', BLATHER, 'Recataloging of ZCatalog at %s terminated' % self.absolute_url(1))
+        if pghandler: pghandler.finish()
 
     def manage_catalogClear(self, REQUEST=None, RESPONSE=None, URL1=None):
         """ clears the whole enchilada """
@@ -460,29 +475,53 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '/manage_catalogIndexes?manage_tabs_message=Index%20Cleared')
 
 
-    def reindexIndex(self, name, REQUEST):
+    def reindexIndex(self, name, REQUEST, pghandler=None):
+        if isinstance(name, str):
+            name = (name,)
+
         paths = self._catalog.uids.keys()
+        num_paths = len(paths)   # inefficient        
+
+        i = 0
+        if pghandler:
+            pghandler.init('reindexing %s' % name, num_paths)
 
         for p in paths:
+            i+=1
+            if pghandler: pghandler.report(i)
+
             obj = self.resolve_path(p)
-            if not obj:
+            if obj is None:
                 obj = self.resolve_url(p, REQUEST)
-            if obj is not None:
+            if obj is None:
+                LOG.error('reindexIndex could not resolve '
+                          'an object from the uid %r.' % p)
+            else:
                 # don't update metadata when only reindexing a single
                 # index via the UI
                 try:
-                    self.catalog_object(obj, p, idxs=[name],
-                                        update_metadata=0)
+                    self.catalog_object(obj, p, idxs=name,
+                                        update_metadata=0, pghandler=pghandler)
                 except TypeError:
                     # Fall back to Zope 2.6.2 interface. This is necessary for
                     # products like CMF 1.4.2 and earlier that subclass from
                     # ZCatalog and don't support the update_metadata argument.
                     # May be removed some day.
-                    from warnings import warn
                     warn('catalog_object interface of %s not up to date'
                          % self.__class__.__name__,
                          DeprecationWarning)
-                    self.catalog_object(obj, p, idxs=[name])
+                    try:
+                        self.catalog_object(obj, p, idxs=name, pghandler=pghandler)
+                    except TypeError:
+                        # Fall back to pre-Zope 2.8 interface where there is no
+                        # 'pghandler' argument.
+                        warn('catalog_object interface of %s not up to date'
+                             % self.__class__.__name__,
+                             DeprecationWarning)
+                        self.catalog_object(obj, p, idxs=name)
+
+        if pghandler:
+            pghandler.finish()
 
     def manage_reindexIndex(self, ids=None, REQUEST=None, RESPONSE=None,
                             URL1=None):
@@ -492,11 +531,9 @@ class ZCatalog(Folder, Persistent, Implicit):
                 message='No items were specified!',
                 action = "./manage_catalogIndexes",)
 
-        if isinstance(ids, str):
-            ids = (ids,)
-
-        for name in ids:
-            self.reindexIndex(name, REQUEST)
+        pgthreshold = self._getProgressThreshold()
+        handler = (pgthreshold > 0) and ZLogHandler(pgthreshold) or None
+        self.reindexIndex(ids, REQUEST, handler)
 
         if REQUEST and RESPONSE:
             RESPONSE.redirect(
@@ -510,7 +547,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         return Splitter.availableSplitters
 
 
-    def catalog_object(self, obj, uid=None, idxs=None, update_metadata=1):
+    def catalog_object(self, obj, uid=None, idxs=None, update_metadata=1, pghandler=None):
         """ wrapper around catalog """
 
         if uid is None:
@@ -534,7 +571,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
         if self.threshold is not None:
             # figure out whether or not to commit a subtransaction.
-            t = id(get_transaction())
+            t = id(transaction.get())
             if t != self._v_transaction:
                 self._v_total = 0
             self._v_transaction = t
@@ -549,9 +586,11 @@ class ZCatalog(Folder, Persistent, Implicit):
             # we should commit a subtransaction if our threshhold is
             # exceeded within the boundaries of the current transaction.
             if self._v_total > self.threshold:
-                get_transaction().commit(1)
+                transaction.savepoint()
                 self._p_jar.cacheGC()
                 self._v_total = 0
+                if pghandler:
+                    pghandler.info('commiting subtransaction')
 
     def uncatalog_object(self, uid):
         """Wrapper around catalog """
@@ -862,52 +901,77 @@ class ZCatalog(Folder, Persistent, Implicit):
                   '%s unchanged.' % (len(fixed), len(removed), unchanged),
           action='./manage_main')
 
-    def manage_convertBTrees(self, threshold=200):
-        """Convert the catalog's data structures to use BTrees package"""
-        assert type(threshold) is type(0)
-        tt=time.time()
-        ct=time.clock()
-        self._catalog._convertBTrees(threshold)
-        tt=time.time()-tt
-        ct=time.clock()-ct
-        return 'Finished conversion in %s seconds (%s cpu)' % (tt, ct)
+    def manage_setProgress(self, pgthreshold=0, RESPONSE=None, URL1=None):
+        """Set parameter to perform logging of reindexing operations very 
+           'pgthreshold' objects
+        """
 
-
-    def manage_convertIndex(self, ids, REQUEST=None, RESPONSE=None, URL1=None):
-        """convert old-style indexes to new-style indexes"""
-
-        from Products.PluginIndexes.KeywordIndex import KeywordIndex
-        from Products.PluginIndexes.FieldIndex import FieldIndex
-        from Products.PluginIndexes.TextIndex import TextIndex
-
-        converted = []
-        for id in ids:
-            idx = self.Indexes[id]
-
-            iface = getattr(idx,'__implements__',None)
-            if iface is None:
-
-                mt = idx.meta_type
-
-                converted.append(id)
-                self.delIndex(id)
-
-                if mt in ('Field Index','Keyword Index'):
-                    self.addIndex(id,mt.replace(' ',''))
-                elif mt == 'Text Index':
-                    # TODO: Lexicon handling to be added
-                    self.addIndex(id,'TextIndex')
-
-        if converted:
+        self.pgthreshold = pgthreshold
+        if RESPONSE:
             RESPONSE.redirect(
-                URL1 +
-                '/manage_main?manage_tabs_message=Indexes%20converted')
-        else:
-            RESPONSE.redirect(
-                URL1 +
-                '/manage_main?'
-                'manage_tabs_message='
-                'No%20indexes%20found%20to%20be%20converted')
+                URL1 + '/manage_main?manage_tabs_message=Catalog%20Changed')
+
+    def _getProgressThreshold(self):
+        if not hasattr(self, 'pgthreshold'):
+            self.pgthreshold = 0
+        return self.pgthreshold
+
+    def manage_convertIndexes(self, REQUEST=None, RESPONSE=None, URL1=None):
+        """Recreate indexes derived from UnIndex because the implementation of
+           __len__ changed in Zope 2.8. Pre-Zope 2.7 installation used to implement
+           __len__ as persistent attribute of the index instance which is totally
+           incompatible with the new extension class implementation based on new-style
+           classes. 
+        """
+
+        LOG.info('Start migration of indexes for %s' % self.absolute_url(1))
+        
+        reindex_ids = []
+
+        for idx in self.Indexes.objectValues():
+            bases = [str(name) for name in idx.__class__.__bases__]
+            found = False
+
+            if idx.meta_type  == 'PathIndex':
+                found = True
+            else:
+                for base in bases:
+                    if 'UnIndex' in base:
+                        found = True
+                        break
+
+            if found:
+                idx_type = idx.meta_type
+                idx_id = idx.getId()
+                LOG.info('processing index %s' % idx_id)
+
+                indexed_attrs = getattr(idx, 'indexed_attrs', None)
+
+                if idx.meta_type == 'DateRangeIndex':
+                    since_field = getattr(idx, '_since_field', None)
+                    until_field = getattr(idx, '_until_field', None)
+
+                self.delIndex(idx.getId())
+                self.addIndex(idx_id, idx_type)
+                new_idx = self.Indexes[idx_id]
+
+                if indexed_attrs:
+                    setattr(new_idx, 'indexed_attrs', indexed_attrs)
+                if idx.meta_type == 'DateRangeIndex':
+                    setattr(new_idx, '_since_field',  since_field)
+                    setattr(new_idx, '_until_field', until_field)
+                reindex_ids.append(idx_id)
+        
+        if reindex_ids:
+            LOG.info('Reindexing %s' % ', '.join(reindex_ids))
+            self.manage_reindexIndex(reindex_ids, REQUEST)
+
+        self._migrated_280 = True
+        LOG.info('Finished migration of indexes for %s' % self.absolute_url(1))
+
+        if RESPONSE:
+            RESPONSE.redirect( URL1 +
+            '/manage_main?manage_tabs_message=Indexes%20converted%20and%20reindexed')
 
 
     #
