@@ -724,7 +724,6 @@ class TALGenerator:
         replace = todo.get("replace")
         condition = todo.get("condition")
         onError = todo.get("onError")
-        define = todo.get("define")
         repldict = todo.get("repldict", {})
         scope = todo.get("scope")
         optTag = todo.get("optional tag")
@@ -790,7 +789,7 @@ class TALGenerator:
             self.emitI18nVariable(varname)
         # Do not test for "msgid is not None", i.e. we only want to test for
         # explicit msgids here.  See comment above.
-        if msgid is not None:
+        if msgid is not None: 
             # in case tal:content, i18n:translate and i18n:name in the
             # same tag insertTranslation opcode has already been
             # emitted
@@ -834,7 +833,7 @@ def _parseI18nAttributes(i18nattrs, attrlist, repldict, position,
     d = {}
     if ';' in i18nattrs:
         i18nattrlist = i18nattrs.split(';')
-        i18nattrlist = [attr.strip().split()
+        i18nattrlist = [attr.strip().split() 
                         for attr in i18nattrlist if attr.strip()]
         for parts in i18nattrlist:
             if len(parts) > 2:
@@ -849,22 +848,39 @@ def _parseI18nAttributes(i18nattrs, attrlist, repldict, position,
             addAttribute(d, attr, msgid, position, xml)
     else:
         i18nattrlist = i18nattrs.split()
-        if len(i18nattrlist) == 2:
+        if len(i18nattrlist) == 1:
+            addAttribute(d, i18nattrlist[0], None, position, xml)
+        elif len(i18nattrlist) == 2:
             staticattrs = [attr[0] for attr in attrlist if len(attr) == 2]
             if (not i18nattrlist[1] in staticattrs) and (
                 not i18nattrlist[1] in repldict):
                 attr, msgid = i18nattrlist
-                addAttribute(d, attr, msgid, position, xml)
+                addAttribute(d, attr, msgid, position, xml)    
             else:
+                import warnings
+                warnings.warn(I18N_ATTRIBUTES_WARNING
+                % (source_file, str(position), i18nattrs)
+                , DeprecationWarning)
                 msgid = None
                 for attr in i18nattrlist:
-                    addAttribute(d, attr, msgid, position, xml)
-        else:
+                    addAttribute(d, attr, msgid, position, xml)    
+        else:    
+            import warnings
+            warnings.warn(I18N_ATTRIBUTES_WARNING
+            % (source_file, str(position), i18nattrs)
+            , DeprecationWarning)
             msgid = None
             for attr in i18nattrlist:
-                addAttribute(d, attr, msgid, position, xml)
+                addAttribute(d, attr, msgid, position, xml)    
     return d
 
+I18N_ATTRIBUTES_WARNING = (
+    'Space separated attributes in i18n:attributes'
+    ' are deprecated (i18n:attributes="value title"). Please use'
+    ' semicolon to separate attributes'
+    ' (i18n:attributes="value; title").'
+    '\nFile %s at row, column %s\nAttributes %s')
+
 def test():
     t = TALGenerator()
     t.pushProgram()
