@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -17,21 +17,21 @@ import select
 import sys
 import threading
 import types
+import logging
 
 import ThreadedAsync
 from ZEO.zrpc import smac
 from ZEO.zrpc.error import ZRPCError, DisconnectedError
-from ZEO.zrpc.log import short_repr, log
 from ZEO.zrpc.marshal import Marshaller
 from ZEO.zrpc.trigger import trigger
-import zLOG
-from ZODB import POSException
+from ZEO.zrpc.log import short_repr, log
+from ZODB.loglevels import BLATHER, TRACE
 
 REPLY = ".reply" # message name used for replies
 ASYNC = 1
 
 class Delay:
-    """Used to delay response to client for synchronous calls
+    """Used to delay response to client for synchronous calls.
 
     When a synchronous call is made and the original handler returns
     without handling the call, it returns a Delay object that prevents
@@ -47,7 +47,7 @@ class Delay:
         self.send_reply(self.msgid, obj)
 
     def error(self, exc_info):
-        log("Error raised in delayed method", zLOG.ERROR, error=exc_info)
+        log("Error raised in delayed method", logging.ERROR, exc_info=True)
         self.return_error(self.msgid, 0, *exc_info[:2])
 
 class MTDelay(Delay):
@@ -87,7 +87,9 @@ class MTDelay(Delay):
 #    own protocol version and the server protocol version, allowing it to
 #    talk to servers using later protocol versions (2.0.2 and higher) as
 #    well:  the effective protocol used will be the lower of the client
-#    and server protocol.
+#    and server protocol.  However, this changed in ZODB 3.3.1 (and
+#    should have changed in ZODB 3.3) because an older server doesn't
+#    support MVCC methods required by 3.3 clients.
 #
 # [Ugly details:  In order to treat the first received message (protocol
 #  handshake) differently than all later messages, both client and server
@@ -174,14 +176,6 @@ class Connection(smac.SizedMessageAsyncConnection, object):
     __super_close = smac.SizedMessageAsyncConnection.close
     __super_setSessionKey = smac.SizedMessageAsyncConnection.setSessionKey
 
-    # Protocol variables:
-    #
-    # oldest_protocol_version -- the oldest protocol version we support
-    # protocol_version -- the newest protocol version we support; preferred
-
-    oldest_protocol_version = "Z200"
-    protocol_version = "Z201"
-
     # Protocol history:
     #
     # Z200 -- Original ZEO 2.0 protocol
@@ -193,21 +187,67 @@ class Connection(smac.SizedMessageAsyncConnection, object):
     #             getAuthProtocol() and scheme-specific authentication methods
     #             getExtensionMethods().
     #             getInvalidations().
+    #
+    # Z303 -- named after the ZODB release 3.3
+    #         Added methods for MVCC:
+    #             loadBefore()
+    #             loadEx()
+    #         A Z303 client cannot talk to a Z201 server, because the latter
+    #         doesn't support MVCC.  A Z201 client can talk to a Z303 server,
+    #         but because (at least) the type of the root object changed
+    #         from ZODB.PersistentMapping to persistent.mapping, the older
+    #         client can't actually make progress if a Z303 client created,
+    #         or ever modified, the root.
+
+    # Protocol variables:
+    # Our preferred protocol.
+    current_protocol = "Z303"
+
+    # If we're a client, an exhaustive list of the server protocols we
+    # can accept.
+    servers_we_can_talk_to = [current_protocol]
+
+    # If we're a server, an exhaustive list of the client protocols we
+    # can accept.
+    clients_we_can_talk_to = ["Z200", "Z201", current_protocol]
+
+    # This is pretty excruciating.  Details:
+    #
+    # 3.3 server 3.2 client
+    #     server sends Z303 to client
+    #     client computes min(Z303, Z201) == Z201 as the protocol to use
+    #     client sends Z201 to server
+    #     OK, because Z201 is in the server's clients_we_can_talk_to
+    #
+    # 3.2 server 3.3 client
+    #     server sends Z201 to client
+    #     client computes min(Z303, Z201) == Z201 as the protocol to use
+    #     Z201 isn't in the client's servers_we_can_talk_to, so client
+    #         raises exception
+    #
+    # 3.3 server 3.3 client
+    #     server sends Z303 to client
+    #     client computes min(Z303, Z303) == Z303 as the protocol to use
+    #     Z303 is in the client's servers_we_can_talk_to, so client
+    #         sends Z303 to server
+    #     OK, because Z303 is in the server's clients_we_can_talk_to
 
     # Client constructor passes 'C' for tag, server constructor 'S'.  This
-    # is used in log messages.
+    # is used in log messages, and to determine whether we can speak with
+    # our peer.
     def __init__(self, sock, addr, obj, tag):
         self.obj = None
         self.marshal = Marshaller()
-        self.closed = 0
+        self.closed = False
         self.peer_protocol_version = None # set in recv_handshake()
 
         assert tag in "CS"
-        tag = "zrpc-conn(%c):" % tag
+        self.tag = tag
+        self.logger = logging.getLogger('ZEO.zrpc.Connection(%c)' % tag)
         if isinstance(addr, types.TupleType):
-            self.log_label = tag + "%s:%d" % addr
+            self.log_label = "(%s:%d) " % addr
         else:
-            self.log_label = tag + str(addr)
+            self.log_label = "(%s) " % addr
 
         # Supply our own socket map, so that we don't get registered with
         # the asyncore socket map just yet.  The initial protocol messages
@@ -227,7 +267,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
         # separate thread.  If thr_async is true, then the asyncore
         # trigger (self.trigger) is used to notify that thread of
         # activity on the current thread.
-        self.thr_async = 0
+        self.thr_async = False
         self.trigger = None
         self._prepare_async()
 
@@ -248,7 +288,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
         # waiting_for_reply is used internally to indicate whether
         # a call is in progress.  setting a session key is deferred
         # until after the call returns.
-        self.waiting_for_reply = 0
+        self.waiting_for_reply = False
         self.delay_sesskey = None
         self.register_object(obj)
 
@@ -283,14 +323,14 @@ class Connection(smac.SizedMessageAsyncConnection, object):
 
     __str__ = __repr__ # Defeat asyncore's dreaded __getattr__
 
-    def log(self, message, level=zLOG.BLATHER, error=None):
-        zLOG.LOG(self.log_label, level, message, error=error)
+    def log(self, message, level=BLATHER, exc_info=False):
+        self.logger.log(level, self.log_label + message, exc_info=exc_info)
 
     def close(self):
         if self.closed:
             return
         self._singleton.clear()
-        self.closed = 1
+        self.closed = True
         self.close_trigger()
         self.__super_close()
 
@@ -300,7 +340,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
             self.trigger.close()
 
     def register_object(self, obj):
-        """Register obj as the true object to invoke methods on"""
+        """Register obj as the true object to invoke methods on."""
         self.obj = obj
 
     # Subclass must implement.  handshake() is called by the constructor,
@@ -324,10 +364,18 @@ class Connection(smac.SizedMessageAsyncConnection, object):
         # Extended by ManagedClientConnection.
         del self.message_input  # uncover normal-case message_input()
         self.peer_protocol_version = proto
-        if self.oldest_protocol_version <= proto <= self.protocol_version:
-            self.log("received handshake %r" % proto, level=zLOG.INFO)
+
+        if self.tag == 'C':
+            good_protos = self.servers_we_can_talk_to
         else:
-            self.log("bad handshake %s" % short_repr(proto), level=zLOG.ERROR)
+            assert self.tag == 'S'
+            good_protos = self.clients_we_can_talk_to
+
+        if proto in good_protos:
+            self.log("received handshake %r" % proto, level=logging.INFO)
+        else:
+            self.log("bad handshake %s" % short_repr(proto),
+                     level=logging.ERROR)
             raise ZRPCError("bad handshake %r" % proto)
 
     def message_input(self, message):
@@ -341,7 +389,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
         if __debug__:
             self.log("recv msg: %s, %s, %s, %s" % (msgid, flags, name,
                                                    short_repr(args)),
-                     level=zLOG.TRACE)
+                     level=TRACE)
         if name == REPLY:
             self.handle_reply(msgid, flags, args)
         else:
@@ -350,7 +398,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
     def handle_reply(self, msgid, flags, args):
         if __debug__:
             self.log("recv reply: %s, %s, %s"
-                     % (msgid, flags, short_repr(args)), level=zLOG.TRACE)
+                     % (msgid, flags, short_repr(args)), level=TRACE)
         self.replies_cond.acquire()
         try:
             self.replies[msgid] = flags, args
@@ -364,22 +412,21 @@ class Connection(smac.SizedMessageAsyncConnection, object):
             raise ZRPCError(msg)
         if __debug__:
             self.log("calling %s%s" % (name, short_repr(args)),
-                     level=zLOG.DEBUG)
+                     level=logging.DEBUG)
 
         meth = getattr(self.obj, name)
         try:
-            self.waiting_for_reply = 1
+            self.waiting_for_reply = True
             try:
                 ret = meth(*args)
             finally:
-                self.waiting_for_reply = 0
+                self.waiting_for_reply = False
         except (SystemExit, KeyboardInterrupt):
             raise
         except Exception, msg:
-            error = sys.exc_info()
-            self.log("%s() raised exception: %s" % (name, msg), zLOG.INFO,
-                     error=error)
-            error = error[:2]
+            self.log("%s() raised exception: %s" % (name, msg), logging.INFO,
+                     exc_info=True)
+            error = sys.exc_info()[:2]
             return self.return_error(msgid, flags, *error)
 
         if flags & ASYNC:
@@ -388,7 +435,8 @@ class Connection(smac.SizedMessageAsyncConnection, object):
                                 (name, short_repr(ret)))
         else:
             if __debug__:
-                self.log("%s returns %s" % (name, short_repr(ret)), zLOG.DEBUG)
+                self.log("%s returns %s" % (name, short_repr(ret)),
+                         logging.DEBUG)
             if isinstance(ret, Delay):
                 ret.set_sender(msgid, self.send_reply, self.return_error)
             else:
@@ -401,14 +449,12 @@ class Connection(smac.SizedMessageAsyncConnection, object):
     def handle_error(self):
         if sys.exc_info()[0] == SystemExit:
             raise sys.exc_info()
-        self.log_error("Error caught in asyncore")
+        self.log("Error caught in asyncore",
+                 level=logging.ERROR, exc_info=True)
         self.close()
 
-    def log_error(self, msg="No error message supplied"):
-        self.log(msg, zLOG.ERROR, error=sys.exc_info())
-
     def check_method(self, name):
-        # XXX Is this sufficient "security" for now?
+        # TODO:  This is hardly "secure".
         if name.startswith('_'):
             return None
         return hasattr(self.obj, name)
@@ -428,7 +474,8 @@ class Connection(smac.SizedMessageAsyncConnection, object):
 
     def return_error(self, msgid, flags, err_type, err_value):
         if flags & ASYNC:
-            self.log_error("Asynchronous call raised exception: %s" % self)
+            self.log("Asynchronous call raised exception: %s" % self,
+                     level=logging.ERROR, exc_info=True)
             return
         if type(err_value) is not types.InstanceType:
             err_value = err_type, err_value
@@ -464,7 +511,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
             self.msgid_lock.release()
         if __debug__:
             self.log("send msg: %d, %d, %s, ..." % (msgid, flags, method),
-                     zLOG.TRACE)
+                     level=TRACE)
         buf = self.marshal.encode(msgid, flags, method, args)
         self.message_output(buf)
         return msgid
@@ -522,14 +569,14 @@ class Connection(smac.SizedMessageAsyncConnection, object):
     # handle IO, possibly in async mode
 
     def _prepare_async(self):
-        self.thr_async = 0
+        self.thr_async = False
         ThreadedAsync.register_loop_callback(self.set_async)
-        # XXX If we are not in async mode, this will cause dead
+        # TODO:  If we are not in async mode, this will cause dead
         # Connections to be leaked.
 
     def set_async(self, map):
         self.trigger = trigger()
-        self.thr_async = 1
+        self.thr_async = True
 
     def is_async(self):
         # Overridden by ManagedConnection
@@ -551,7 +598,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
         """Invoke asyncore mainloop and wait for reply."""
         if __debug__:
             self.log("wait(%d), async=%d" % (msgid, self.is_async()),
-                     level=zLOG.TRACE)
+                     level=TRACE)
         if self.is_async():
             self._pull_trigger()
 
@@ -569,7 +616,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
                     del self.replies[msgid]
                     if __debug__:
                         self.log("wait(%d): reply=%s" %
-                                 (msgid, short_repr(reply)), level=zLOG.TRACE)
+                                 (msgid, short_repr(reply)), level=TRACE)
                     return reply
                 if self.is_async():
                     self.replies_cond.wait(10.0)
@@ -579,13 +626,13 @@ class Connection(smac.SizedMessageAsyncConnection, object):
                         try:
                             if __debug__:
                                 self.log("wait(%d): asyncore.poll(%s)" %
-                                         (msgid, delay), level=zLOG.TRACE)
+                                         (msgid, delay), level=TRACE)
                             asyncore.poll(delay, self._singleton)
                             if delay < 1.0:
                                 delay += delay
                         except select.error, err:
                             self.log("Closing.  asyncore.poll() raised %s."
-                                     % err, level=zLOG.BLATHER)
+                                     % err, level=BLATHER)
                             self.close()
                     finally:
                         self.replies_cond.acquire()
@@ -602,7 +649,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
     def poll(self):
         """Invoke asyncore mainloop to get pending message out."""
         if __debug__:
-            self.log("poll(), async=%d" % self.is_async(), level=zLOG.TRACE)
+            self.log("poll(), async=%d" % self.is_async(), level=TRACE)
         if self.is_async():
             self._pull_trigger()
         else:
@@ -611,7 +658,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
     def pending(self, timeout=0):
         """Invoke mainloop until any pending messages are handled."""
         if __debug__:
-            self.log("pending(), async=%d" % self.is_async(), level=zLOG.TRACE)
+            self.log("pending(), async=%d" % self.is_async(), level=TRACE)
         if self.is_async():
             return
         # Inline the asyncore poll() function to know whether any input
@@ -642,9 +689,9 @@ class Connection(smac.SizedMessageAsyncConnection, object):
                 # loop is only intended to make sure all incoming data is
                 # returned.
 
-                # XXX What if the server sends a lot of invalidations,
-                # such that pending never finishes?  Seems unlikely, but
-                # not impossible.
+                # Insecurity:  What if the server sends a lot of
+                # invalidations, such that pending never finishes?  Seems
+                # unlikely, but possible.
                 timeout = 0
             if r:
                 try:
@@ -673,7 +720,7 @@ class ManagedServerConnection(Connection):
 
     def handshake(self):
         # Send the server's preferred protocol to the client.
-        self.message_output(self.protocol_version)
+        self.message_output(self.current_protocol)
 
     def close(self):
         self.obj.notifyDisconnected()
@@ -721,7 +768,7 @@ class ManagedClientConnection(Connection):
 
     def handshake(self):
         # The client waits to see the server's handshake.  Outgoing messages
-        # aere queued for the duration.  The client will send its own
+        # are queued for the duration.  The client will send its own
         # handshake after the server's handshake is seen, in recv_handshake()
         # below.  It will then send any messages queued while waiting.
         assert self.queue_output # the constructor already set this
@@ -729,7 +776,7 @@ class ManagedClientConnection(Connection):
     def recv_handshake(self, proto):
         # The protocol to use is the older of our and the server's preferred
         # protocols.
-        proto = min(proto, self.protocol_version)
+        proto = min(proto, self.current_protocol)
 
         # Restore the normal message_input method, and raise an exception
         # if the protocol version is too old.
@@ -752,6 +799,7 @@ class ManagedClientConnection(Connection):
 
     def close_trigger(self):
         # the manager should actually close the trigger
+        # TODO: what is that comment trying to say?  What 'manager'?
         del self.trigger
 
     def set_async(self, map):
@@ -765,13 +813,13 @@ class ManagedClientConnection(Connection):
         if not self.thr_async and self.mgr.thr_async:
             assert self.mgr.trigger is not None, \
                    "manager (%s) has no trigger" % self.mgr
-            self.thr_async = 1
+            self.thr_async = True
             self.trigger = self.mgr.trigger
             return 1
         return 0
 
     def is_async(self):
-        # XXX could the check_mgr_async() be avoided on each test?
+        # TODO: could the check_mgr_async() be avoided on each test?
         if self.thr_async:
             return 1
         return self.check_mgr_async()
