@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -13,65 +13,116 @@
 ##############################################################################
 """Handy standard storage machinery
 
-$Id: BaseStorage.py,v 1.34.4.4 2005/04/22 19:36:07 tim_one Exp $
+$Id: BaseStorage.py 30116 2005-04-22 20:53:25Z tim_one $
 """
 import cPickle
+import threading
 import time
+import logging
 from struct import pack as _structpack, unpack as _structunpack
 
-import ThreadLock
-import zLOG
-from ZODB import bpthread
+from persistent.TimeStamp import TimeStamp
+
 from ZODB import POSException
-from ZODB.TimeStamp import TimeStamp
-from ZODB.UndoLogCompatible import UndoLogCompatible
 from ZODB.utils import z64, oid_repr
+from ZODB.UndoLogCompatible import UndoLogCompatible
+
+log = logging.getLogger("ZODB.BaseStorage")
 
 class BaseStorage(UndoLogCompatible):
+    """Abstract base class that supports storage implementations.
+
+    A subclass must define the following methods:
+    load()
+    close()
+    cleanup()
+    lastSerial()
+    lastTransaction()
+
+    It must override these hooks:
+    _begin()
+    _vote()
+    _abort()
+    _finish()
+    _clear_temp()
+
+    If it stores multiple revisions, it should implement
+    loadSerial()
+    loadBefore()
+    iterator()
+
+    If the subclass wants to implement undo, it should implement the
+    multiple revision methods and:
+    loadSerial()
+    undo()
+    undoInfo()
+    undoLog()
+
+    If the subclass wants to implement versions, it must implement:
+    abortVersion()
+    commitVersion()
+    modifiedInVersion()
+    versionEmpty()
+    versions()
+
+    Each storage will have two locks that are accessed via lock
+    acquire and release methods bound to the instance.  (Yuck.)
+    _lock_acquire / _lock_release (reentrant)
+    _commit_lock_acquire / _commit_lock_release
+
+    The commit lock is acquired in tpc_begin() and released in
+    tpc_abort() and tpc_finish().  It is never acquired with the other
+    lock held.
+
+    The other lock appears to protect _oid and _transaction and
+    perhaps other things.  It is always held when load() is called, so
+    presumably the load() implementation should also acquire the lock.
+    """
     _transaction=None # Transaction that is being committed
-    _serial=z64       # Transaction serial number
     _tstatus=' '      # Transaction status, used for copying data
-    _is_read_only = 0
+    _is_read_only = False
 
     def __init__(self, name, base=None):
         self.__name__= name
-        zLOG.LOG(self.__class__.__name__, zLOG.DEBUG,
-                 "create storage %s" % self.__name__)
+        log.debug("create storage %s", self.__name__)
 
         # Allocate locks:
-        l=ThreadLock.allocate_lock()
-        self._lock_acquire=l.acquire
-        self._lock_release=l.release
-        l=bpthread.allocate_lock()
-        self._commit_lock_acquire=l.acquire
-        self._commit_lock_release=l.release
+        l = threading.RLock()
+        self._lock_acquire = l.acquire
+        self._lock_release = l.release
+        l = threading.Lock()
+        self._commit_lock_acquire = l.acquire
+        self._commit_lock_release = l.release
 
         t=time.time()
         t=self._ts=apply(TimeStamp,(time.gmtime(t)[:5]+(t%60,)))
-        self._serial=`t`
+        self._tid = `t`
 
         # ._oid is the highest oid in use (0 is always in use -- it's
         # a reserved oid for the root object).  Our new_oid() method
         # increments it by 1, and returns the result.  It's really a
         # 64-bit integer stored as an 8-byte big-endian string.
         if base is None:
-            self._oid = '\0\0\0\0\0\0\0\0'
+            self._oid = z64
         else:
             self._oid = base._oid
 
     def abortVersion(self, src, transaction):
         if transaction is not self._transaction:
             raise POSException.StorageTransactionError(self, transaction)
-        return []
+        return self._tid, []
 
     def commitVersion(self, src, dest, transaction):
         if transaction is not self._transaction:
             raise POSException.StorageTransactionError(self, transaction)
-        return []
+        return self._tid, []
 
     def close(self):
         pass
 
+    def cleanup(self):
+        pass
+
     def sortKey(self):
         """Return a string that can be used to sort storage instances.
 
@@ -87,7 +138,7 @@ class BaseStorage(UndoLogCompatible):
     def getSize(self):
         return len(self)*300 # WAG!
 
-    def history(self, oid, version, length=1):
+    def history(self, oid, version, length=1, filter=None):
         pass
 
     def modifiedInVersion(self, oid):
@@ -130,9 +181,6 @@ class BaseStorage(UndoLogCompatible):
     def supportsUndo(self):
         return 0
 
-    def supportsTransactionalUndo(self):
-        return 0
-
     def supportsVersions(self):
         return 0
 
@@ -178,13 +226,13 @@ class BaseStorage(UndoLogCompatible):
                 now = time.time()
                 t = TimeStamp(*(time.gmtime(now)[:5] + (now % 60,)))
                 self._ts = t = t.laterThan(self._ts)
-                self._serial = `t`
+                self._tid = `t`
             else:
                 self._ts = TimeStamp(tid)
-                self._serial = tid
+                self._tid = tid
 
             self._tstatus = status
-            self._begin(self._serial, user, desc, ext)
+            self._begin(self._tid, user, desc, ext)
         finally:
             self._lock_release()
 
@@ -208,16 +256,23 @@ class BaseStorage(UndoLogCompatible):
         pass
 
     def tpc_finish(self, transaction, f=None):
+        # It's important that the storage calls the function we pass
+        # while it still has its lock.  We don't want another thread
+        # to be able to read any updated data until we've had a chance
+        # to send an invalidation message to all of the other
+        # connections!
+
         self._lock_acquire()
         try:
             if transaction is not self._transaction:
                 return
             try:
                 if f is not None:
-                    f()
+                    f(self._tid)
                 u, d, e = self._ude
-                self._finish(self._serial, u, d, e)
+                self._finish(self._tid, u, d, e)
                 self._clear_temp()
+                return self._tid
             finally:
                 self._ude = None
                 self._transaction = None
@@ -230,7 +285,7 @@ class BaseStorage(UndoLogCompatible):
         """
         pass
 
-    def undo(self, transaction_id):
+    def undo(self, transaction_id, txn):
         if self._is_read_only:
             raise POSException.ReadOnlyError()
         raise POSException.UndoError, 'non-undoable transaction'
@@ -261,6 +316,48 @@ class BaseStorage(UndoLogCompatible):
         raise POSException.Unsupported, (
             "Retrieval of historical revisions is not supported")
 
+    def loadBefore(self, oid, tid):
+        """Return most recent revision of oid before tid committed."""
+
+        # Unsure: Is it okay for loadBefore() to return current data?
+        # There doesn't seem to be a good reason to forbid it, even
+        # though the typical use of this method will never find
+        # current data.  But maybe we should call it loadByTid()?
+
+        n = 2
+        start_time = None
+        end_time = None
+        while start_time is None:
+            # The history() approach is a hack, because the dict
+            # returned by history() doesn't contain a tid.  It
+            # contains a serialno, which is often the same, but isn't
+            # required to be.  We'll pretend it is for now.
+
+            # A second problem is that history() doesn't say anything
+            # about whether the transaction status.  If it falls before
+            # the pack time, we can't honor the MVCC request.
+
+            # Note: history() returns the most recent record first.
+
+            # TODO: The filter argument to history() only appears to be
+            # supported by FileStorage.  Perhaps it shouldn't be used.
+            L = self.history(oid, "", n, lambda d: not d["version"])
+            if not L:
+                return
+            for d in L:
+                if d["serial"] < tid:
+                    start_time = d["serial"]
+                    break
+                else:
+                    end_time = d["serial"]
+            if len(L) < n:
+                break
+            n *= 2
+        if start_time is None:
+            return None
+        data = self.loadSerial(oid, start_time)
+        return data, start_time, end_time
+
     def getExtensionMethods(self):
         """getExtensionMethods
 
@@ -277,7 +374,7 @@ class BaseStorage(UndoLogCompatible):
         """Copy transactions from another storage.
 
         This is typically used for converting data from one storage to
-        another.  `other' must have an .iterator() method.
+        another.  `other` must have an .iterator() method.
         """
         _ts=None
         ok=1
@@ -320,9 +417,10 @@ class BaseStorage(UndoLogCompatible):
             self.tpc_begin(transaction, tid, transaction.status)
             for r in transaction:
                 oid=r.oid
-                if verbose: print oid_repr(oid), r.version, len(r.data)
+                if verbose:
+                    print oid_repr(oid), r.version, len(r.data)
                 if restoring:
-                    self.restore(oid, r.serial, r.data, r.version,
+                    self.restore(oid, r.tid, r.data, r.version,
                                  r.data_txn, transaction)
                 else:
                     pre=preget(oid, None)
