@@ -28,21 +28,5 @@ class PrefixTest(unittest.TestCase):
         for equal in ("", "/", "/def", "/a/b", "/a/b/c", "/a/b/c/d"):
             self.assertEqual(p2, equal)
 
-    def test_pre327_compat(self):
-        string = "a/b/"
-        p = Prefix(string)
-        # Prior to 3.2.7, Prefix had only a value attribute.  Make sure
-        # an old-format Prefix gets converted to new-format automagically.
-        length, path = p.length, p.path
-        del p.length, p.path
-        p.value = len(string), string
-        self.assertEqual(repr(p), "Prefix('" + string + "')")
-        self.assertEqual(p.length, length)
-        self.assertEqual(p.path, path)
-
-        # Ensure the pre-3.2.7 .value attribute gets created too.
-        p = Prefix(string)
-        self.assertEqual(p.value, (len(string), string))
-
 def test_suite():
     return unittest.makeSuite(PrefixTest)
